"""TyPySetup - Interactive Python environment setup CLI for VSCode."""

__version__ = "1.0.0"
__author__ = "Miguel Muniz"
__email__ = "miguimuniz@gmail.com"
__license__ = "MIT"

from .main import app

__all__ = ["app", "__version__"]
